@@ -6,8 +6,12 @@
 #' @param constraints An optional character specifying under what
 #' constraints ("purposes") we are requesting the values.
 #' For instance, on systems where multicore processing is not supported
-#' (i.e. Windows), using `constrains = "multicore"` will force a
+#' (i.e. Windows), using `constraints = "multicore"` will force a
 #' single core to be reported.
+#' Using `constraints = "connections"`, will append `"connections"` to
+#' the `methods` argument.
+#' It is possible to specify multiple constraints, e.g.
+#' `constraints = c("connections", "multicore")`.
 #'
 #' @param methods A character vector specifying how to infer the number
 #' of available cores.
@@ -15,8 +19,10 @@
 #' @param na.rm If TRUE, only non-missing settings are considered/returned.
 #'
 #' @param logical Passed to
-#' \code{\link[parallel]{detectCores}(logical = logical)}, which, if supported,
-#' returns the number of logical CPUs (TRUE) or physical CPUs/cores (FALSE).
+#' \code{\link[parallel]{detectCores}(logical = logical)}, which,
+#' _if supported_, returns the number of logical CPUs (TRUE) or physical
+#' CPUs/cores (FALSE).
+#' At least as of R 4.2.2, `detectCores()` this argument on Linux.
 #' This argument is only if argument `methods` includes `"system"`.
 #'
 #' @param default The default number of cores to return if no non-missing
@@ -40,13 +46,19 @@
 #'  \item `"system"` -
 #'    Query \code{\link[parallel]{detectCores}(logical = logical)}.
 #'
+#'  \item `"/proc/self/status"` -
+#'    Query \code{Cpus_allowed_list} of `/proc/self/status`.
+#'
 #'  \item `"cgroups.cpuset"` -
-#'    On Unix, query control group (cgroup) value \code{cpuset.set}.
+#'    On Unix, query control group (cgroup v1) value \code{cpuset.set}.
 #'
 #'  \item `"cgroups.cpuquota"` -
-#'    On Unix, query control group (cgroup) value
+#'    On Unix, query control group (cgroup v1) value
 #'    \code{cpu.cfs_quota_us} / \code{cpu.cfs_period_us}.
 #'
+#'  \item `"cgroups2.cpu.max"` -
+#'    On Unix, query control group (cgroup v2) values \code{cpu.max}.
+#'
 #'  \item `"nproc"` -
 #'    On Unix, query system command \code{nproc}.
 #'
@@ -64,12 +76,30 @@
 #'    instance \code{\link[=mclapply]{mclapply}()} of the \pkg{parallel}
 #'    package.
 #'
+#'  \item `"connections"` -
+#'    Query the current number of available R connections per
+#'    [freeConnections()].  This is the maximum number of socket-based
+#'    **parallel** cluster nodes that are possible launch, because each
+#'    one needs its own R connection.
+#'    The exception is when `freeConnections()` is zero, then `1L` is
+#'    still returned, because `availableCores()` should always return a
+#'    positive integer.
+#'
 #'  \item `"BiocParallel"` -
-#'    Query environment variables \env{BIOCPARALLEL_WORKER_NUMBER} (integer),
-#'    which is defined and used by **BiocParallel** (>= 1.27.2), and
-#'    \env{BBS_HOME} (logical) used by the Bioconductor Build System. If the
-#'    former is set, this is the number of cores considered.  If the latter
-#'    is set, then a maximum of 4 cores is considered.
+#'    Query environment variable \env{BIOCPARALLEL_WORKER_NUMBER} (integer),
+#'    which is defined and used by **BiocParallel** (>= 1.27.2).
+#'    If the former is set, this is the number of cores considered.
+#'
+#'  \item `"_R_CHECK_LIMIT_CORES_"` -
+#'    Query environment variable \env{_R_CHECK_LIMIT_CORES_} (logical or
+#'    `"warn"`) used by `R CMD check` and set to true by
+#'    `R CMD check --as-cran`. If set to a non-false value, then a maximum
+#'    of 2 cores is considered.
+#'
+#'  \item `"Bioconductor"` -
+#'    Query environment variable \env{IS_BIOC_BUILD_MACHINE} (logical)
+#'    used by the Bioconductor (>= 3.16) build and check system. If set to
+#'    true, then a maximum of 4 cores is considered.
 #'
 #'  \item `"LSF"` - 
 #'    Query Platform Load Sharing Facility (LSF) environment variable
@@ -92,7 +122,8 @@
 #'
 #'  \item `"SGE"` -
 #'    Query Sun Grid Engine/Oracle Grid Engine/Son of Grid Engine (SGE)
-#'    environment variable \env{NSLOTS}.
+#'    and Univa Grid Engine (UGE)/Altair Grid Engine (AGE) environment
+#'    variable \env{NSLOTS}.
 #'    An example of a job submission that results in this is
 #'    `qsub -pe smp 2` (or `qsub -pe by_node 2`), which
 #'    requests two cores on a single machine.
@@ -106,6 +137,8 @@
 #'    If \env{SLURM_CPUS_PER_TASK} is not set, then it will fall back to
 #'    use \env{SLURM_CPUS_ON_NODE} if the job is a single-node job
 #'    (\env{SLURM_JOB_NUM_NODES} is 1), e.g. `sbatch --ntasks=2 hello.sh`.
+#'    To make sure all tasks are assign to a single node, specify
+#'    `--nodes=1`, e.g. `sbatch --nodes=1 --ntasks=16 hello.sh`.
 #'
 #'  \item `"custom"` -
 #'    If option
@@ -187,19 +220,14 @@
 #'
 #' @importFrom parallel detectCores
 #' @export
-availableCores <- function(constraints = NULL, methods = getOption2("parallelly.availableCores.methods", c("system", "cgroups.cpuset", "cgroups.cpuquota", "nproc", "mc.cores", "BiocParallel", "_R_CHECK_LIMIT_CORES_", "LSF", "PJM", "PBS", "SGE", "Slurm", "fallback", "custom")), na.rm = TRUE, logical = getOption2("parallelly.availableCores.logical", TRUE), default = c(current = 1L), which = c("min", "max", "all"), omit = getOption2("parallelly.availableCores.omit", 0L)) {
-  ## Local functions
-  getenv <- function(name, mode = "integer") {
-    value <- trim(getEnvVar2(name, default = NA_character_))
-    storage.mode(value) <- mode
-    value
-  } # getenv()
+availableCores <- function(constraints = NULL, methods = getOption2("parallelly.availableCores.methods", c("system", "/proc/self/status", "cgroups.cpuset", "cgroups.cpuquota", "cgroups2.cpu.max", "nproc", "mc.cores", "BiocParallel", "_R_CHECK_LIMIT_CORES_", "Bioconductor", "LSF", "PJM", "PBS", "SGE", "Slurm", "fallback", "custom")), na.rm = TRUE, logical = getOption2("parallelly.availableCores.logical", TRUE), default = c(current = 1L), which = c("min", "max", "all"), omit = getOption2("parallelly.availableCores.omit", 0L)) {
+  stop_if_not(
+    is.null(constraints) || is.character(constraints), !anyNA(constraints)
+  )
 
-  getopt <- function(name, mode = "integer") {
-    value <- getOption2(name, default = NA_integer_)
-    storage.mode(value) <- mode
-    value
-  } # getopt()
+  if ("connections" %in% constraints) {
+    methods <- unique(c(methods, "connections"))
+  }
 
   which <- match.arg(which, choices = c("min", "max", "all"))
   stop_if_not(length(default) == 1, is.finite(default), default >= 1L)
@@ -214,97 +242,27 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
     method <- methods[kk]
     if (method == "Slurm") {
       ## Number of cores assigned by Slurm
-
-      ## The assumption is that the following works regardless of
-      ## number of nodes requested /HB 2020-09-18
-      ## Example: --cpus-per-task={n}
-      n <- getenv("SLURM_CPUS_PER_TASK")
-      if (is.na(n)) {
-        ## Example: --nodes={nnodes} (defaults to 1, short: -N {nnodes})
-        ## From 'man sbatch':
-        ## SLURM_JOB_NUM_NODES (and SLURM_NNODES for backwards compatibility)
-        ## Total number of nodes in the job's resource allocation.
-        nnodes <- getenv("SLURM_JOB_NUM_NODES")
-        if (is.na(nnodes)) nnodes <- getenv("SLURM_NNODES")
-        if (is.na(nnodes)) nnodes <- 1L  ## Can this happen? /HB 2020-09-18
-
-        if (nnodes == 1L) {
-          ## Example: --nodes=1 --ntasks={n} (short: -n {n})
-          ## IMPORTANT: 'SLURM_CPUS_ON_NODE' appears to be rounded up when nodes > 1.
-          ## Example 1: With --nodes=2 --cpus-per-task=3 we see SLURM_CPUS_ON_NODE=4
-          ## although SLURM_CPUS_PER_TASK=3. 
-          ## Example 2: With --nodes=2 --ntasks=7, we see SLURM_CPUS_ON_NODE=6,
-          ## SLURM_JOB_CPUS_PER_NODE=6,2, no SLURM_CPUS_PER_TASK, and
-          ## SLURM_TASKS_PER_NODE=5,2.
-          ## Conclusions: We can only use 'SLURM_CPUS_ON_NODE' for nnodes = 1.
-          n <- getenv("SLURM_CPUS_ON_NODE")
-        } else {
-          ## Parse `SLURM_TASKS_PER_NODE`
-          nodecounts <- getenv("SLURM_TASKS_PER_NODE", mode = "character")
-          if (!is.na(nodecounts)) {
-            ## Examples:
-            ## SLURM_TASKS_PER_NODE=5,2
-            ## SLURM_TASKS_PER_NODE=2(x2),1(x3)  # Source: 'man sbatch'
-            n <- slurm_expand_nodecounts(nodecounts)
-            if (any(is.na(n))) next
-
-            ## ASSUMPTION: We assume that it is the first component on the list that
-            ## corresponds to the current machine. /HB 2021-03-05
-            n <- n[1]
-          }
-        }
-      }
-
-      ## TODO?: Can we validate above assumptions/results? /HB 2020-09-18
-      if (FALSE && !is.na(n)) {
-        ## Is any of the following useful?
-
-        ## Example: --ntasks={ntasks} (no default, short: -n {ntasks})
-        ## From 'man sbatch':
-        ## SLURM_NTASKS (and SLURM_NPROCS for backwards compatibility)
-        ## Same as -n, --ntasks
-        ntasks <- getenv("SLURM_NTASKS")
-        if (is.na(ntasks)) ntasks <- getenv("SLURM_NPROCS")
-      }
+      n <- availableCoresSlurm()
     } else if (method == "PBS") {
-      ## Number of cores assigned by TORQUE/PBS
-      n <- getenv("PBS_NUM_PPN")
-      if (is.na(n)) {
-        ## PBSPro sets 'NCPUS' but not 'PBS_NUM_PPN'
-        n <- getenv("NCPUS")
-      }
+      n <- availableCoresPBS()
     } else if (method == "SGE") {
-      ## Number of cores assigned by Sun/Oracle Grid Engine (SGE)
-      n <- getenv("NSLOTS")
+      n <- availableCoresSGE()
     } else if (method == "LSF") {
-      ## Number of slots assigned by LSF
-      n <- getenv("LSB_DJOB_NUMPROC")
+      n <- availableCoresLSF()
     } else if (method == "PJM") {
-      ## Number of slots assigned by Fujitsu Technical Computing Suite
-      ## We choose to call this job scheduler "PJM" based on the prefix
-      ## it's environment variables use.
-      ## PJM_VNODE_CORE: e.g. pjsub -L vnode-core=8
-      ## "This environment variable is set only when virtual nodes
-      ##  are allocated, and it is not set when nodes are allocated."
-      n <- getenv("PJM_VNODE_CORE")
-      if (is.na(n)) {
-        ## PJM_PROC_BY_NODE: e.g. pjsub -L vnode-core=8
-        ## "Maximum number of processes that are generated per node by
-        ##  an MPI program. However, if a single node (node=1) or virtual
-        ##  node (vnode=1) is allocated and the mpi option of the pjsub
-        ##  command is not specified, this environment variable is not set."
-        n <- getenv("PJM_PROC_BY_NODE")
-      }
+      n <- availableCoresPJM()
     } else if (method == "mc.cores") {
       ## Number of cores by option defined by 'parallel' package
-      n <- getopt("mc.cores")
+      n <- getopt_int("mc.cores")
       if (!is.na(n) && n == 0) n <- 1L  ## Because options(mc.cores = 0) may be set
     } else if (method == "mc.cores+1") {
       ## Number of cores by option defined by 'parallel' package
-      n <- getopt("mc.cores") + 1L
+      n <- getopt_int("mc.cores") + 1L
+    } else if (method == "connections") {
+      ## Number of available connections, which are needed by PSOCK clusters
+      n <- freeConnections()
     } else if (method == "BiocParallel") {
-      n <- getenv("BIOCPARALLEL_WORKER_NUMBER")
-      if (nzchar(Sys.getenv("BBS_HOME"))) n <- min(n, 4L, na.rm = TRUE)
+      n <- getenv_int("BIOCPARALLEL_WORKER_NUMBER")
     } else if (method == "_R_CHECK_LIMIT_CORES_") {
       ## A flag set by R CMD check for constraining number of
       ## cores allowed to be use in package tests.  Here we
@@ -315,20 +273,54 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
       chk <- tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_", ""))
       chk <- (nzchar(chk) && (chk != "false"))
       n <- if (chk) 2L else NA_integer_
+    } else if (method == "Bioconductor") {
+      n <- NA_integer_
+      ## Bioconductor (>= 3.16)
+      use <- Sys.getenv("IS_BIOC_BUILD_MACHINE", NA_character_)
+      if (isTRUE(as.logical(use))) n <- min(n, 4L, na.rm = TRUE)
+      ## Legacy: Bioconductor (<= 3.15)
+      if (is.na(n)) {
+        use <- Sys.getenv("BBS_HOME", NA_character_)
+        if (isTRUE(as.logical(use))) n <- min(n, 4L, na.rm = TRUE)
+      }
     } else if (method == "system") {
       ## Number of cores available according to parallel::detectCores()
       n <- detectCores(logical = logical)
+    } else if (method == "/proc/self/status") {
+      pathname <- "/proc/self/status"
+      if (file_test("-f", pathname)) {
+        bfr <- readLines(pathname, warn = FALSE)
+        bfr <- grep("^Cpus_allowed_list:", bfr, value = TRUE)
+        if (length(bfr) == 1) {
+          bfr <- sub("^Cpus_allowed_list:\t", "", bfr)
+          if (nzchar(bfr)) {
+            bfr <- slurm_expand_nodelist(sprintf("[%s]", bfr))
+            n <- length(bfr)
+          }
+        }
+      }
     } else if (method == "cgroups.cpuset") {
-      ## Number of cores according to Unix Cgroups CPU set
-      n <- length(getCGroupsCpuSet())
+      ## Number of cores according to Unix cgroups v1 CPU set
+      n <- length(getCGroups1CpuSet())
       if (n == 0L) n <- NA_integer_
     } else if (method == "cgroups.cpuquota") {
-      ## Number of cores according to Unix Cgroups CPU quota
-      n <- getCGroupsCpuQuota()
-      if (!is.na(n)) {
+      ## Number of cores according to Unix cgroups v1 CPU quota
+      n <- getCGroups1CpuQuota()
+      if (is.na(n)) {
+        n <- NA_integer_
+      } else {
         n <- as.integer(floor(n + 0.5))
 	if (n == 0L) n <- 1L  ## If CPU quota < 0.5, round up to one CPU
       }
+    } else if (method == "cgroups2.cpu.max") {
+      ## Number of cores according to Unix cgroups v2 CPU max quota
+      n <- getCGroups2CpuMax()
+      if (is.na(n)) {
+        n <- NA_integer_
+      } else {
+        n <- as.integer(floor(n + 0.5))
+	if (n == 0L) n <- 1L  ## If CPU max quota < 0.5, round up to one CPU
+      }
     } else if (method == "nproc") {
       ## Number of cores according to Unix 'nproc'
       n <- getNproc()
@@ -345,19 +337,19 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
         on.exit(options(oopts))
         fcn()
       })
-      n <- as.integer(n)
       if (length(n) != 1L) {
         stop("Function specified by option 'parallelly.availableCores.custom' does not a single value")
       }
+      n <- as.integer(n)
     } else {
       ## covr: skip=3
       ## Fall back to querying option and system environment variable
       ## with the given name
-      n <- getopt(method)
-      if (is.na(n)) n <- getenv(method)
+      n <- getopt_int(method)
+      if (is.na(n)) n <- getenv_int(method)
     }
     ncores[kk] <- n
-  }
+  } ## for (kk in seq_along(methods))
 
   ## Validate settings
   ncoresT <- ncores[!is.na(ncores)]
@@ -410,8 +402,8 @@ availableCores <- function(constraints = NULL, methods = getOption2("parallelly.
     }
   }
 
-  if (!is.null(constraints)) {
-    if (constraints == "multicore") {
+  if (length(constraints) > 0L) {
+    if ("multicore" %in% constraints) {
       ## SPECIAL: On some OSes such as Windows, multicore processing
       ## is not supported.  If so, we should override all values to
       ## to reflect that only a single core is available
@@ -476,3 +468,169 @@ getNproc <- function(ignore = c("OMP_NUM_THREADS", "OMP_THREAD_LIMIT")) {
   
   NA_integer_
 }
+
+
+checkNumberOfLocalWorkers <- function(workers) {
+  if (inherits(workers, "AsIs")) return()
+  
+  limits <- getOption("parallelly.maxWorkers.localhost", c(1.0, 3.0))
+  if (length(limits) == 0) return()
+
+  ## FIXME: Temporarily, ignore _R_CHECK_LIMIT_CORES_ limits
+  ## This will give a few packages time to be fixed. /HB 2024-02-09
+  ignore <- c("_R_CHECK_LIMIT_CORES_")
+  ignore <- getOption("parallelly.maxWorkers.localhost.ignore", ignore)
+  if (length(ignore) > 0) {
+    methods <- eval(formals(availableCores)$methods)
+    methods <- setdiff(methods, ignore)
+    ncores <- availableCores(methods = methods)
+  } else {
+    ncores <- availableCores()
+  }
+  reason <- names(ncores)
+  if (is.null(reason)) reason <- "N/A"
+  
+  rho <- workers / ncores
+
+  ## Produce an error?
+  if (length(limits) >= 2) {
+    if (rho > limits[2]) {
+      msg <- sprintf("Attempting to set up %d localhost parallel workers with only %d CPU cores available for this R process (per %s), which could result in a %.0f%% load", workers, ncores, sQuote(reason), 100 * workers / ncores)
+      msg <- sprintf("%s. The hard limit is set to %.0f%%", msg, 100 * limits[2])
+      msg <- sprintf("%s. Overusing the CPUs has negative impact on the current R process, but also on all other processes of yours and others running on the same machine", msg)
+      msg <- sprintf("%s. See help(\"parallelly.options\", package = \"parallelly\") for how to override the soft and hard limits", msg)
+      stop(msg)
+    }
+  }
+  
+  ## Warn?
+  if (rho > limits[1]) {
+    msg <- sprintf("Careful, you are setting up %d localhost parallel workers with only %d CPU cores available for this R process (per %s), which could result in a %.0f%% load", workers, ncores, sQuote(reason), 100 * workers / ncores)
+    msg <- sprintf("%s. The soft limit is set to %.0f%%", msg, 100 * limits[1])
+    msg <- sprintf("%s. Overusing the CPUs has negative impact on the current R process, but also on all other processes of yours and others running on the same machine", msg)
+    msg <- sprintf("%s. See help(\"parallelly.options\", package = \"parallelly\") for how to override the soft and hard limits", msg)
+    warning(msg)
+  }
+} ## checkNumberOfLocalWorkers()
+
+  
+# --------------------------------------------------------------------------
+# Utility functions
+# --------------------------------------------------------------------------
+getenv_int <- function(name, mode = "integer") {
+  value <- trim(getEnvVar2(name, default = NA_character_))
+  storage.mode(value) <- mode
+  value
+} # getenv_int()
+
+getopt_int <- function(name, mode = "integer") {
+  value <- getOption2(name, default = NA_integer_)
+  storage.mode(value) <- mode
+  value
+} # getopt_int()
+
+
+# --------------------------------------------------------------------------
+# High-Performance Compute (HPC) Schedulers
+# --------------------------------------------------------------------------
+## Number of slots assigned by LSF
+availableCoresLSF <- function() {      
+  n <- getenv_int("LSB_DJOB_NUMPROC")
+  n
+}
+
+
+## Number of cores assigned by TORQUE/PBS
+availableCoresPBS <- function() {      
+  n <- getenv_int("PBS_NUM_PPN")
+  if (is.na(n)) {
+    ## PBSPro sets 'NCPUS' but not 'PBS_NUM_PPN'
+    n <- getenv_int("NCPUS")
+  }
+  n
+}
+
+
+## Number of slots assigned by Fujitsu Technical Computing Suite
+## We choose to call this job scheduler "PJM" based on the prefix
+## it's environment variables use.
+availableCoresPJM <- function() {      
+  ## PJM_VNODE_CORE: e.g. pjsub -L vnode-core=8
+  ## "This environment variable is set only when virtual nodes
+  ##  are allocated, and it is not set when nodes are allocated."
+  n <- getenv_int("PJM_VNODE_CORE")
+  if (is.na(n)) {
+    ## PJM_PROC_BY_NODE: e.g. pjsub -L vnode-core=8
+    ## "Maximum number of processes that are generated per node by
+    ##  an MPI program. However, if a single node (node=1) or virtual
+    ##  node (vnode=1) is allocated and the mpi option of the pjsub
+    ##  command is not specified, this environment variable is not set."
+    n <- getenv_int("PJM_PROC_BY_NODE")
+  }
+  n
+}
+
+
+## Number of cores assigned by Oracle/Son/Sun/Univa Grid Engine (SGE/UGE)
+availableCoresSGE <- function() {      
+  n <- getenv_int("NSLOTS")
+  n
+}
+
+
+## Number of cores assigned by Slurm
+availableCoresSlurm <- function() {
+  ## The assumption is that the following works regardless of
+  ## number of nodes requested /HB 2020-09-18
+  ## Example: --cpus-per-task={n}
+  n <- getenv_int("SLURM_CPUS_PER_TASK")
+  if (is.na(n)) {
+    ## Example: --nodes={nnodes} (defaults to 1, short: -N {nnodes})
+    ## From 'man sbatch':
+    ## SLURM_JOB_NUM_NODES (and SLURM_NNODES for backwards compatibility)
+    ## Total number of nodes in the job's resource allocation.
+    nnodes <- getenv_int("SLURM_JOB_NUM_NODES")
+    if (is.na(nnodes)) nnodes <- getenv_int("SLURM_NNODES")
+    if (is.na(nnodes)) nnodes <- 1L  ## Can this happen? /HB 2020-09-18
+
+    if (nnodes == 1L) {
+      ## Example: --nodes=1 --ntasks={n} (short: -n {n})
+      ## IMPORTANT: 'SLURM_CPUS_ON_NODE' appears to be rounded up when nodes > 1.
+      ## Example 1: With --nodes=2 --cpus-per-task=3 we see SLURM_CPUS_ON_NODE=4
+      ## although SLURM_CPUS_PER_TASK=3. 
+      ## Example 2: With --nodes=2 --ntasks=7, we see SLURM_CPUS_ON_NODE=6,
+      ## SLURM_JOB_CPUS_PER_NODE=6,2, no SLURM_CPUS_PER_TASK, and
+      ## SLURM_TASKS_PER_NODE=5,2.
+      ## Conclusions: We can only use 'SLURM_CPUS_ON_NODE' for nnodes = 1.
+      n <- getenv_int("SLURM_CPUS_ON_NODE")
+    } else {
+      ## Parse `SLURM_TASKS_PER_NODE`
+      nodecounts <- getenv_int("SLURM_TASKS_PER_NODE", mode = "character")
+      if (!is.na(nodecounts)) {
+        ## Examples:
+        ## SLURM_TASKS_PER_NODE=5,2
+        ## SLURM_TASKS_PER_NODE=2(x2),1(x3)  # Source: 'man sbatch'
+        n <- slurm_expand_nodecounts(nodecounts)
+        if (anyNA(n)) return(NA_real_)
+
+        ## ASSUMPTION: We assume that it is the first component on the list that
+        ## corresponds to the current machine. /HB 2021-03-05
+        n <- n[1]
+      }
+    }
+  }
+
+  ## TODO?: Can we validate above assumptions/results? /HB 2020-09-18
+  if (FALSE && !is.na(n)) {
+    ## Is any of the following useful?
+
+    ## Example: --ntasks={ntasks} (no default, short: -n {ntasks})
+    ## From 'man sbatch':
+    ## SLURM_NTASKS (and SLURM_NPROCS for backwards compatibility)
+    ## Same as -n, --ntasks
+    ntasks <- getenv_int("SLURM_NTASKS")
+    if (is.na(ntasks)) ntasks <- getenv_int("SLURM_NPROCS")
+  }
+
+  n
+} ## availableCoresSlurm()
