@@ -1,3 +1,5 @@
+.packageExclude <- c("MPI", "PSOCK")
+
 ## covr: skip=all
 .onLoad <- function(libname, pkgname) {
   debug <- isTRUE(as.logical(getEnvVar2("R_PARALLELLY_DEBUG", "FALSE")))
@@ -30,7 +32,9 @@
     ## Lower the risk for port clashes by using a large pool of random ports.
     ## The default 11000:11999 tend to fail occassionally on CRAN but also
     ## locally.
-    Sys.setenv(R_PARALLELLY_RANDOM_PORTS = "20000:39999")
+    if (!nzchar(Sys.getenv("R_PARALLELLY_RANDOM_PORTS", ""))) {
+      Sys.setenv(R_PARALLELLY_RANDOM_PORTS = "20000:39999")
+    }
 
     ## Not all CRAN servers have _R_CHECK_LIMIT_CORES_ set [REF?]. Setting it
     ## to 'TRUE' when unset, will better emulate CRAN submission checks.
@@ -54,4 +58,15 @@
     envs <- sprintf("- %s=%s", names(envs), sQuote(envs))
     mdebug(paste(c("parallelly-specific environment variables:", envs), collapse = "\n"))
   }
+
+  registerClusterTypes()
+
+  ## R CMD build
+  register_vignette_engine_during_build_only(pkgname)
+}
+
+
+#' @useDynLib "parallelly", .registration = TRUE, .fixes = "C_"
+.onUnload <- function(libpath) {
+  library.dynam.unload(.packageName, libpath)
 }
